# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""Generic input/output devices."""


class LUMPDevice:
    """Devices using the LEGO UART Messaging Protocol."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the device is connected.
        """
        pass

    def read(self, mode):
        """Reads values from a given mode.

        Arguments:
            mode (``int``): Device mode.

        Returns:
            ``tuple``: Values read from the sensor.
        """
        pass

    def write(self, mode, values):
        """Writes values to the sensor. Only selected sensors and modes support
        this.

        Arguments:
            mode (``int``): Device mode.
            data (``tuple``): Values to be written.
        """
        pass


class Ev3devSensor:
    """Read values of an ev3dev-compatible sensor."""

    sensor_index = 0
    """Index of the ev3dev sysfs `lego-sensor`_ class."""

    port_index = 0
    """Index of the ev3dev sysfs `lego-port`_ class."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the device is connected.
        """
        pass

    def read(self, mode):
        """Reads values at a given mode.

        Arguments:
            mode (``str``): `Mode name`_.

        Returns:
            ``tuple``: Values read from the sensor.
        """
        pass


class AnalogSensor:
    """Generic or custom analog sensor."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.
        """
        pass

    def voltage(self):
        """Measures analog voltage.

        Returns:
            :ref:`voltage`: Analog voltage.
        """
        pass

    def resistance(self):
        """Measures resistance.

        This value is only meaningful if the analog device is a passive load
        such as a resistor or thermistor.

        Returns:
            :ref:`resistance: Ω <voltage>`: Resistance of the analog device.
        """
        pass

    def active(self):
        """Sets sensor to active mode. This sets pin 5 of the sensor
        port to `high`.

        This is used in some analog
        sensors to control a switch. For example, if you use the NXT Light
        Sensor as a custom analog sensor, this method will turn the light on.
        From then on, ``voltage()`` returns the raw reflected light value.
        """
        pass

    def passive(self):
        """Sets sensor to passive mode. This sets pin 5 of the sensor
        port to `low`.

        This is used in some analog
        sensors to control a switch. For example, if you use the NXT Light
        Sensor as a custom analog sensor, this method will turn the light off.
        From then on, ``voltage()`` returns the raw ambient light value.
        """
        pass


class I2CDevice:
    """Generic or custom I2C device."""

    def __init__(self, port, address):
        """

        Arguments:
            port (Port): Port to which the device is connected.
            address(int): I2C address of the client device. See
                :ref:`I2C Addresses <i2caddress>`.
        """
        pass

    def read(self, reg, length=1):
        """Reads bytes, starting at a given register.

        Arguments:
            reg (``int``): Register at which to begin
                reading: 0--255 or 0x00--0xFF.
            length (``int``): How many bytes to read.

        Returns:
            ``bytes``: Bytes returned from the device.
        """
        pass

    def write(self, reg, data=None):
        """Writes bytes, starting at a given register.

        Arguments:
            reg (``int``): Register at which to begin
                writing: 0--255 or 0x00--0xFF.
            data (``bytes``): Bytes to be written.
        """
        pass


class UARTDevice:
    """Generic UART device."""

    def __init__(self, port, baudrate, timeout=None):
        """

        Arguments:
            port (Port): Port to which the device is connected.
            baudrate (int): Baudrate of the UART device.
            timeout (:ref:`time`): How long to wait
                during :meth:`.read` before giving up. If you choose ``None``,
                it will wait forever.
        """
        pass

    def read(self, length=1):
        """Reads a given number of bytes from the buffer.

        Your program will wait until the requested number of bytes are
        received. If this takes longer than ``timeout``, the ``ETIMEDOUT``
        exception is raised.

        Arguments:
            length (``int``): How many bytes to read.

        Returns:
            ``bytes``: Bytes returned from the device.
        """
        pass

    def read_all(self):
        """Reads all bytes from the buffer.

        Returns:
            ``bytes``: Bytes returned from the device.
        """
        pass

    def write(self, data):
        """Writes bytes.

        Arguments:
            data (``bytes``): Bytes to be written.
        """
        pass

    def waiting(self):
        """Gets how many bytes are still waiting to be read.

        Returns:
            ``int``: Number of bytes in the buffer.
        """
        pass

    def clear(self):
        """Empties the buffer."""
        pass



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

class Color:
    BLACK: Color
    BLUE: Color
    GREEN: Color
    YELLOW: Color
    RED: Color
    WHITE: Color
    BROWN: Color
    ORANGE: Color
    PURPLE: Color

class Port:
    A: Port
    B: Port
    C: Port
    D: Port
    S1: Port
    S2: Port
    S3: Port
    S4: Port

class Stop:
    COAST: Stop
    BRAKE: Stop
    HOLD: Stop

class Direction:
    CLOCKWISE: Direction
    COUNTERCLOCKWISE: Direction

class Button:
    LEFT_DOWN: Button
    DOWN: Button
    RIGHT_DOWN: Button
    LEFT: Button
    CENTER: Button
    RIGHT: Button
    LEFT_UP: Button
    UP: Button
    BEACON: Button
    RIGHT_UP: Button



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""Constant parameters/arguments for the Pybricks API."""

from enum import Enum as _Enum


class _PybricksEnumMeta(type(_Enum)):
    def __dir__(cls):
        yield '__class__'
        yield '__name__'
        for member in cls:
            yield member.name


class _PybricksEnum(_Enum, metaclass=_PybricksEnumMeta):
    def __dir__(self):
        yield '__class__'
        for member in type(self):
            yield member.name

    def __str__(self):
        return '{}.{}'.format(type(self).__name__, self.name)

    def __repr__(self):
        return str(self)


class Color(_PybricksEnum):
    """Light or surface color.

    .. data:: BLACK
    .. data:: BLUE
    .. data:: GREEN
    .. data:: YELLOW
    .. data:: RED
    .. data:: WHITE
    .. data:: BROWN
    .. data:: ORANGE
    .. data:: PURPLE
    """

    BLACK = 1
    BLUE = 2
    GREEN = 3
    YELLOW = 4
    RED = 5
    WHITE = 6
    BROWN = 7
    ORANGE = 8
    PURPLE = 9


class Port(_PybricksEnum):
    """Port on the programmable brick or hub."""

    # Generic motor/sensor ports
    A = ord('A')
    B = ord('B')
    C = ord('C')
    D = ord('D')
    E = ord('E')
    F = ord('F')

    # NXT/EV3 sensor ports
    S1 = ord('1')
    S2 = ord('2')
    S3 = ord('3')
    S4 = ord('4')


class Stop(_PybricksEnum):
    """Action after the motor stops: coast, brake, or hold.

    .. data:: COAST

        Let the motor move freely.

    .. data:: BRAKE

        Passively resist small external forces.

    .. data:: HOLD

        Keep controlling the motor to hold it at the commanded angle. This is
        only available on motors with encoders.
    """

    COAST = 0
    BRAKE = 1
    HOLD = 2


class Direction(_PybricksEnum):
    """Rotational direction for positive speed or angle values.

    .. data:: CLOCKWISE

        A positive speed value should make the motor move clockwise.

    .. data:: COUNTERCLOCKWISE

        A positive speed value should make the motor move counterclockwise.

    +--------------------------------+-------------------+-----------------+
    | ``positive_direction =``       | Positive speed:   | Negative speed: |
    +================================+===================+=================+
    | ``Direction.CLOCKWISE``        | clockwise         | counterclockwise|
    +--------------------------------+-------------------+-----------------+
    | ``Direction.COUNTERCLOCKWISE`` | counterclockwise  | clockwise       |
    +--------------------------------+-------------------+-----------------+
    """

    CLOCKWISE = 0
    COUNTERCLOCKWISE = 1


class Button(_PybricksEnum):
    """Buttons on a brick or remote:

    .. data:: LEFT_DOWN
    .. data:: DOWN
    .. data:: RIGHT_DOWN
    .. data:: LEFT
    .. data:: CENTER
    .. data:: RIGHT
    .. data:: LEFT_UP
    .. data:: UP
    .. data:: BEACON
    .. data:: RIGHT_UP

    +-----------+----------+-----------+
    |           |          |           |
    | LEFT_UP   |UP/BEACON | RIGHT_UP  |
    |           |          |           |
    +-----------+----------+-----------+
    |           |          |           |
    | LEFT      |  CENTER  | RIGHT     |
    |           |          |           |
    +-----------+----------+-----------+
    |           |          |           |
    | LEFT_DOWN |   DOWN   | RIGHT_DOWN|
    |           |          |           |
    +-----------+----------+-----------+
    """

    LEFT_DOWN = 1
    DOWN = 2
    RIGHT_DOWN = 3
    LEFT = 4
    CENTER = 5
    RIGHT = 6
    LEFT_UP = 7
    UP = 8
    BEACON = 8
    RIGHT_UP = 9


class Axis(_PybricksEnum):
    """Unit axes of a coordinate system.

    .. data:: X
    .. data:: Y
    .. data:: Z

    Some methods let you measure along all axes at once:

    .. data:: ALL
    """
    X = (1, 0, 0)
    Y = (0, 1, 0)
    Z = (0, 0, 1)
    ALL = None


class Side(_PybricksEnum):
    """Sides or face of a device such as a hub or a sensor. Such devices are
    usually shaped like a rectangular box with six of the following sides:

    .. data:: TOP
    .. data:: BOTTOM
    .. data:: FRONT
    .. data:: BACK
    .. data:: LEFT
    .. data:: RIGHT
    """

    RIGHT = 6
    FRONT = 0
    TOP = 8
    LEFT = 4
    BACK = 5
    BOTTOM = 2



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from typing import Iterable, List, Optional, Tuple, Union, overload

from .parameters import Button, Color, Direction, Stop, Port
from .media.ev3dev import SoundFile

class DCMotor:
    def __init__(
        self, port: Port, positive_direction: Direction = Direction.CLOCKWISE
    ): ...
    def dc(self, duty: int) -> None: ...
    def stop(self) -> None: ...
    def brake(self) -> None: ...

class Control:
    @overload
    def limits(self) -> Tuple[int, int, int]: ...
    @overload
    def limits(
        self,
        speed: Optional[int] = None,
        acceleration: Optional[int] = None,
        actuation: Optional[int] = None,
    ) -> None: ...
    def pid(self) -> Tuple[int, int, int, int, int, int]: ...
    @overload
    def pid(
        self,
        kp: Optional[int] = None,
        ki: Optional[int] = None,
        kd: Optional[int] = None,
        integral_range: Optional[int] = None,
        integral_rate: Optional[int] = None,
        feed_forward: Optional[int] = None,
    ) -> None: ...
    @overload
    def target_tolerances(self) -> Tuple[int, int]: ...
    @overload
    def target_tolerances(
        self, speed: Optional[int] = None, position: Optional[int] = None
    ) -> None: ...
    @overload
    def stall_tolerances(self) -> Tuple[int, int]: ...
    @overload
    def stall_tolerances(
        self, speed: Optional[int] = None, time: Optional[int] = None
    ) -> None: ...
    def trajectory(
        self,
    ) -> Tuple[int, int, int, int, int, int, int, int, int, int, int, int]: ...
    def stalled(self) -> bool: ...
    def done(self) -> bool: ...

class Motor(DCMotor):
    control: Control
    def __init__(
        self,
        port: Port,
        positive_direction: Direction = Direction.CLOCKWISE,
        gears: Optional[Union[List[int], List[List[int]]]] = None,
    ): ...
    def angle(self) -> int: ...
    def speed(self) -> int: ...
    def reset_angle(self, angle: int) -> None: ...
    def hold(self) -> None: ...
    def run(self, speed: int) -> None: ...
    def run_time(
        self, speed: int, time: int, then: Stop = Stop.HOLD, wait: bool = True
    ) -> None: ...
    def run_angle(
        self, speed: int, rotation_angle: int, then: Stop = Stop.HOLD, wait: bool = True
    ) -> None: ...
    def run_target(
        self, speed: int, target_angle: int, then: Stop = Stop.HOLD, wait: bool = True
    ) -> None: ...
    def run_until_stalled(
        self, speed: int, then: Stop = Stop.COAST, duty_limit: Optional[int] = None
    ) -> int: ...
    def track_target(self, target_angle: int) -> None: ...

class Speaker:
    def beep(self, frequency: int = 500, duration: int = 100) -> None: ...
    def play_notes(self, notes: Iterable[str], tempo: int = 120) -> None: ...
    def play_file(self, file_name: Union[SoundFile, str]) -> None: ...
    def say(self, text: str) -> None: ...
    def set_speech_options(
        self,
        language: Optional[str] = None,
        voice: Optional[str] = None,
        speed: Optional[int] = None,
        pitch: Optional[int] = None,
    ): ...
    def set_volume(self, volume: int, which: str = "_all_") -> None: ...

class Light:
    def on(self, brightness: int = 100) -> None: ...
    def off(self) -> None: ...

class ColorLight:
    def on(self, color: Optional[Color]) -> None: ...
    def off(self) -> None: ...
    def rgb(self, red: int, green: int, blue: int) -> None: ...

class KeyPad:
    def pressed(self) -> List[Button]:...

class Battery:
    def voltage(self) -> int: ...
    def current(self) -> int: ...



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from typing import Any, Literal, Optional, Union, overload

from ..parameters import Color

class Image:
    @overload
    def __init__(self, /, source: Union[Image, ImageFile, str]): ...
    @overload
    def __init__(
        self, /, source: Image, sub: Literal[True], x1: int, y1: int, x2: int, y2: int
    ): ...
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...
    def clear(self) -> None: ...
    def draw_pixel(self, x: int, y: int, color: Color = Color.BLACK) -> None: ...
    def draw_line(
        self,
        x1: int,
        y1: int,
        x2: int,
        y2: int,
        width: int = 1,
        color: Color = Color.BLACK,
    ) -> None: ...
    def draw_box(
        self,
        x1: int,
        y1: int,
        x2: int,
        y2: int,
        r: int = 0,
        fill: bool = False,
        color: Color = Color.BLACK,
    ) -> None: ...
    def draw_circle(
        self, x: int, y: int, r: int, fill: bool = False, color: Color = Color.BLACK
    ) -> None: ...
    def draw_image(
        self,
        x: int,
        y: int,
        source: Union[Image, ImageFile, str],
        transparent: Optional[Color] = None,
    ) -> None: ...
    def load_image(self, source: Union[Image, ImageFile, str]) -> None: ...
    def draw_text(
        self,
        x: int,
        y: int,
        text: str,
        text_color: Color = Color.BLACK,
        background_color: Optional[Color] = None,
    ) -> None: ...
    def print(self, *args: Any, sep: str = " ", end: str = "\n") -> None: ...
    def set_font(self, font: Font) -> None: ...
    @staticmethod
    def empty(width: int = 178, height: int = 128) -> Image: ...
    def save(self, filename: str) -> None: ...

class Font:
    DEFAULT: Font
    def __init__(
        self,
        family: Optional[str] = None,
        size: int = 12,
        bold: bool = False,
        monospace: bool = False,
        lang: Optional[str] = None,
        script: Optional[str] = None,
    ): ...
    @property
    def family(self) -> str: ...
    @property
    def style(self) -> str: ...
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...
    def text_width(self, text: str) -> int: ...
    def text_height(self, text: str) -> int: ...

class SoundFile:
    SHOUTING: SoundFile
    CHEERING: SoundFile
    CRYING: SoundFile
    OUCH: SoundFile
    LAUGHING_2: SoundFile
    SNEEZING: SoundFile
    SMACK: SoundFile
    BOING: SoundFile
    BOO: SoundFile
    UH_OH: SoundFile
    SNORING: SoundFile
    KUNG_FU: SoundFile
    FANFARE: SoundFile
    CRUNCHING: SoundFile
    MAGIC_WAND: SoundFile
    LAUGHING_1: SoundFile
    LEFT: SoundFile
    BACKWARDS: SoundFile
    RIGHT: SoundFile
    OBJECT: SoundFile
    COLOR: SoundFile
    FLASHING: SoundFile
    ERROR: SoundFile
    ERROR_ALARM: SoundFile
    DOWN: SoundFile
    FORWARD: SoundFile
    ACTIVATE: SoundFile
    SEARCHING: SoundFile
    TOUCH: SoundFile
    UP: SoundFile
    ANALYZE: SoundFile
    STOP: SoundFile
    DETECTED: SoundFile
    TURN: SoundFile
    START: SoundFile
    MORNING: SoundFile
    EV3: SoundFile
    GO: SoundFile
    GOOD_JOB: SoundFile
    OKEY_DOKEY: SoundFile
    GOOD: SoundFile
    NO: SoundFile
    THANK_YOU: SoundFile
    YES: SoundFile
    GAME_OVER: SoundFile
    OKAY: SoundFile
    SORRY: SoundFile
    BRAVO: SoundFile
    GOODBYE: SoundFile
    HI: SoundFile
    HELLO: SoundFile
    MINDSTORMS: SoundFile
    LEGO: SoundFile
    FANTASTIC: SoundFile
    SPEED_IDLE: SoundFile
    SPEED_DOWN: SoundFile
    SPEED_UP: SoundFile
    BROWN: SoundFile
    GREEN: SoundFile
    BLACK: SoundFile
    WHITE: SoundFile
    RED: SoundFile
    BLUE: SoundFile
    YELLOW: SoundFile
    TICK_TACK: SoundFile
    HORN_1: SoundFile
    BACKING_ALERT: SoundFile
    MOTOR_IDLE: SoundFile
    AIR_RELEASE: SoundFile
    AIRBRAKE: SoundFile
    RATCHET: SoundFile
    MOTOR_STOP: SoundFile
    HORN_2: SoundFile
    LASER: SoundFile
    SONAR: SoundFile
    MOTOR_START: SoundFile
    INSECT_BUZZ_2: SoundFile
    ELEPHANT_CALL: SoundFile
    SNAKE_HISS: SoundFile
    DOG_BARK_2: SoundFile
    DOG_WHINE: SoundFile
    INSECT_BUZZ_1: SoundFile
    DOG_SNIFF: SoundFile
    T_REX_ROAR: SoundFile
    INSECT_CHIRP: SoundFile
    DOG_GROWL: SoundFile
    SNAKE_RATTLE: SoundFile
    DOG_BARK_1: SoundFile
    CAT_PURR: SoundFile
    EIGHT: SoundFile
    SEVEN: SoundFile
    SIX: SoundFile
    FOUR: SoundFile
    TEN: SoundFile
    ONE: SoundFile
    TWO: SoundFile
    THREE: SoundFile
    ZERO: SoundFile
    FIVE: SoundFile
    NINE: SoundFile
    READY: SoundFile
    CONFIRM: SoundFile
    GENERAL_ALERT: SoundFile
    CLICK: SoundFile
    OVERPOWER: SoundFile

class ImageFile:
    RIGHT: ImageFile
    FORWARD: ImageFile
    ACCEPT: ImageFile
    QUESTION_MARK: ImageFile
    STOP_1: ImageFile
    LEFT: ImageFile
    DECLINE: ImageFile
    THUMBS_DOWN: ImageFile
    BACKWARD: ImageFile
    NO_GO: ImageFile
    WARNING: ImageFile
    STOP_2: ImageFile
    THUMBS_UP: ImageFile
    EV3: ImageFile
    EV3_ICON: ImageFile
    TARGET: ImageFile
    BOTTOM_RIGHT: ImageFile
    BOTTOM_LEFT: ImageFile
    EVIL: ImageFile
    CRAZY_2: ImageFile
    KNOCKED_OUT: ImageFile
    PINCHED_RIGHT: ImageFile
    WINKING: ImageFile
    DIZZY: ImageFile
    DOWN: ImageFile
    TIRED_MIDDLE: ImageFile
    MIDDLE_RIGHT: ImageFile
    SLEEPING: ImageFile
    MIDDLE_LEFT: ImageFile
    TIRED_RIGHT: ImageFile
    PINCHED_LEFT: ImageFile
    PINCHED_MIDDLE: ImageFile
    CRAZY_1: ImageFile
    NEUTRAL: ImageFile
    AWAKE: ImageFile
    UP: ImageFile
    TIRED_LEFT: ImageFile
    ANGRY: ImageFile



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""Generic cross-platform module for typical devices like lights, displays,
speakers, and batteries."""

from .parameters import Direction, Stop, Axis


class DCMotor:
    """Generic class to control simple motors without rotation sensors, such
    as train motors."""

    def __init__(self, port,
                 positive_direction=Direction.CLOCKWISE):
        """

        Arguments:
            port (Port): Port to which the motor is connected.
            positive_direction (Direction): Which direction the motor should
                turn when you give a positive duty cycle value.
        """
        pass

    def dc(self, duty):
        """Rotates the motor at a given duty cycle (also known as "power").

        Arguments:
            duty (:ref:`percentage`): The duty cycle (-100.0 to 100).
        """
        pass

    def stop(self):
        """Stops the motor and lets it spin freely.

        The motor gradually stops due to friction."""
        pass

    def brake(self):
        """Passively brakes the motor.

        The motor stops due to friction, plus the voltage that
        is generated while the motor is still moving."""
        pass


class Control:
    """Class to interact with PID controller and settings.

        .. data:: scale

            Scaling factor between the controlled integer variable
            and the physical output. For example, for a single
            motor this is the number of encoder pulses per degree of rotation.
    """

    def limits(self, speed, acceleration, actuation):
        """Configures the maximum speed, acceleration, and actuation.

        If no arguments are given, this will return the current values.

        Arguments:
            speed (:ref:`speed` or :ref:`linspeed`):
                Maximum speed. All speed commands will be capped to this value.
            acceleration (:ref:`acceleration` or :ref:`linacceleration`):
                Maximum acceleration.
            actuation (:ref:`percentage`):
                Maximum actuation as percentage of absolute maximum.
        """
        pass

    def pid(self, kp, ki, kd, integral_range, integral_rate, feed_forward):
        """Gets or sets the PID values for position and speed control.

        If no arguments are given, this will return the current values.

        Arguments:
            kp (int): Proportional position (or integral speed) control
                constant.
            ki (int): Integral position control constant.
            kd (int): Derivative position (or proportional speed) control
                constant.
            integral_range (:ref:`angle` or :ref:`distance`): Region around
                the target angle or distance, in which integral control errors
                are accumulated.
            integral_rate (:ref:`speed` or :ref:`linspeed`): Maximum rate at
                which the error integral is allowed to grow.
            feed_forward (:ref:`percentage`):
                This adds a feed forward signal to the PID feedback signal, in
                the direction of the speed reference. This value is expressed
                as a percentage of the absolute maximum duty cycle.
        """
        pass

    def target_tolerances(self, speed, position):
        """Gets or sets the tolerances that say when a maneuver is done.

        If no arguments are given, this will return the current values.

        Arguments:
            speed (:ref:`speed` or :ref:`linspeed`): Allowed deviation
                from zero speed before motion is considered complete.
            position (:ref:`angle` or :ref:`distance`): Allowed
                deviation from the target before motion is considered
                complete.
        """
        pass

    def stall_tolerances(self, speed, time):
        """Gets or sets stalling tolerances.

        If no arguments are given, this will return the current values.

        Arguments:
            speed (:ref:`speed` or :ref:`linspeed`): If the controller
                cannot reach this speed for some ``time`` even with maximum
                actuation, it is stalled.
            time (:ref:`time`): How long the controller has to be below this
                minimum ``speed`` before we say it is stalled.
        """
        pass

    def stalled(self):
        """Checks if the controller is currently stalled.

        A controller is stalled when it cannot reach the target speed or
        position, even with the maximum actuation signal.

        Returns:
            bool: ``True`` if the controller is stalled, ``False`` if not.
        """
        pass

    def done(self):
        """Checks if an ongoing command or maneuver is done.

        Returns:
            bool: ``True`` if the command is done, ``False`` if not.
        """
        pass


class Motor(DCMotor):
    """Generic class to control motors with built-in rotation sensors."""

    control = Control()
    """The motors use PID control to accurately track the speed and
    angle targets that you specify. You can change its behavior through the
    ``control`` attribute of the motor. See :ref:`control` for an overview
    of available methods."""

    def __init__(self, port,
                 positive_direction=Direction.CLOCKWISE,
                 gears=None):
        """

        Arguments:
            port (Port): Port to which the motor is connected.
            positive_direction (Direction): Which direction the motor should
                turn when you give a positive speed value or
                angle.
            gears (list):
                List of gears linked to the motor.

                For example: ``[12, 36]`` represents a gear train with a
                12-tooth and a 36-tooth gear. Use a list of lists for multiple
                gear trains, such as ``[[12, 36], [20, 16, 40]]``.

                When you specify a gear train, all motor commands and settings
                are automatically adjusted to account for the resulting gear
                ratio.  The motor direction remains unchanged by this.
        """
        pass

    def angle(self):
        """Gets the rotation angle of the motor.

        Returns:
            :ref:`angle`: Motor angle.

        """
        pass

    def speed(self):
        """Gets the speed of the motor.

        Returns:
            :ref:`speed`: Motor speed.

        """
        pass

    def reset_angle(self, angle):
        """Sets the accumulated rotation angle of the motor to a desired value.

        Arguments:
            angle (:ref:`angle`): Value to which the angle should be reset.
        """
        pass

    def hold(self):
        """Stops the motor and actively holds it at its current angle."""
        pass

    def run(self, speed):
        """Runs the motor at a constant speed.

        The motor accelerates to the given speed and keeps running at this
        speed until you give a new command.

        Arguments:
            speed (:ref:`speed`): Speed of the motor.
        """
        pass

    def run_time(self, speed, time, then=Stop.HOLD, wait=True):
        """Runs the motor at a constant speed for a given amount of time.

        The motor accelerates to the given speed, keeps running at this speed,
        and then decelerates. The total maneuver lasts for exactly the given
        amount of ``time``.

        Arguments:
            speed (:ref:`speed`): Speed of the motor.
            time (:ref:`time`): Duration of the maneuver.
            then (Stop): What to do after coming to a standstill.
            wait (bool): Wait for the maneuver to complete before continuing
                         with the rest of the program.
        """
        pass

    def run_angle(self, speed, rotation_angle, then=Stop.HOLD, wait=True):
        """Runs the motor at a constant speed by a given angle.

        Arguments:
            speed (:ref:`speed`): Speed of the motor.
            rotation_angle (:ref:`angle`): Angle by which the motor should
                                           rotate.
            then (Stop): What to do after coming to a standstill.
            wait (bool): Wait for the maneuver to complete before continuing
                         with the rest of the program.
        """
        pass

    def run_target(self, speed, target_angle, then=Stop.HOLD, wait=True):
        """Runs the motor at a constant speed towards a
        given target angle.

        The direction of rotation is automatically selected based on the target
        angle. It does matter if ``speed`` is positive or negative.

        Arguments:
            speed (:ref:`speed`): Speed of the motor.
            target_angle (:ref:`angle`): Angle that the motor should
                                         rotate to.
            then (Stop): What to do after coming to a standstill.
            wait (bool): Wait for the motor to reach the target
                         before continuing with the rest of the
                         program.
        """
        pass

    def run_until_stalled(self, speed, then=Stop.COAST, duty_limit=None):
        """Runs the motor at a constant speed until it stalls.

        Arguments:
            speed (:ref:`speed`): Speed of the motor.
            then (Stop): What to do after coming to a standstill.
            duty_limit (:ref:`percentage`): Torque limit during this
                command. This is useful to avoid applying the full motor
                torque to a geared or lever mechanism.

        Returns:
            :ref:`angle`: Angle at which the motor becomes stalled.
        """
        pass

    def track_target(self, target_angle):
        """Tracks a target angle. This is similar to :meth:`.run_target`, but
        the usual smooth acceleration is skipped: it will move to the target
        angle as fast as possible. This method is useful if you want to
        continuously change the target angle.

        Arguments:
            target_angle (:ref:`angle`): Target angle that the motor should
                                         rotate to.

        """
        pass

    def dc(self, duty):
        """Rotates the motor at a given duty cycle (also known as "power").

        This method lets you use a motor just like a simple DC motor.

        Arguments:
            duty (:ref:`percentage`): The duty cycle (-100.0 to 100).
        """


class Speaker:
    """Plays beeps and sounds using a speaker."""

    def beep(self, frequency=500, duration=100):
        """Play a beep/tone.

        Arguments:
            frequency (:ref:`frequency`):
                Frequency of the beep. Frequencies below 100
                are treated as 100.
            duration (:ref:`time`):
                Duration of the beep. If the duration is less
                than 0, then the method returns immediately and the frequency
                play continues to play indefinitely.
        """
        pass

    def play_notes(self, notes, tempo=120):
        """Plays a sequence of musical notes.

        For example, you can play: ``['C4/4', 'C4/4', 'G4/4', 'G4/4']``.

        Arguments:
            notes (iter):
                A sequence of notes to be played (see format below).
            tempo (int):
                Beats per minute where a quarter note is one beat.
        """
        pass

    def play_file(self, file_name):
        """Plays a sound file.

        Arguments:
            file_name (str):
                Path to the sound file, including the file extension.
        """

        pass

    def say(self, text):
        """Says a given text string.

        You can configure the language and voice of the text using
        :meth:`set_speech_options`.

        Arguments:
            text (str): What to say.
        """

        pass

    def set_speech_options(self, language=None, voice=None, speed=None, pitch=None):
        """Configures speech settings used by the :meth:`say` method.

        Any option that is set to ``None`` will not be changed. If an option
        is set to an invalid value :meth:`say` will use the default value
        instead.

        Arguments:
            language (str):
                Language of the text. For example, you can choose ``'en'``
                (English) or ``'de'`` (German). A list of all available
                languages is given below.
            voice (str):
                The voice to use. For example, you can choose ``'f1'`` (female
                voice variant 1) or ``'m3'`` (male voice variant 3). A list of
                all available voices is given below.
            speed (int):
                Number of words per minute.
            pitch (int):
                Pitch (0 to 99). Higher numbers make the voice higher pitched
                and lower numbers make the voice lower pitched.
        """
        pass

    def set_volume(self, volume, which='_all_'):
        """Sets the speaker volume.

        Arguments:
            volume (:ref:`percentage`):
                Volume of the speaker.
            which (str):
                Which volume to set. ``'Beep'`` sets the volume for
                :meth:`beep` and :meth:`play_notes`. ``'PCM'`` sets the volume
                for :meth:`play_file` and :meth:`say`. ``'_all_'`` sets both
                at the same time.
        """
        pass


class Light:
    """Control a single-color light."""

    def on(self, brightness=100):
        """Turn on the light at the specified brightness.

        Arguments:
            brightness (:ref:`brightness`):
                Brightness of the light.
        """

    def off(self):
        """Turns off the light."""
        pass


class ColorLight:
    """Control a multi-color light."""

    def on(self, color):
        """Turns on the light at the specified color.

        Arguments:
            color (Color): Color of the light. The light turns off if you
                           choose ``None`` or a color that is not available.
        """
        pass

    def off(self):
        """Turns off the light."""
        pass

    def rgb(self, red, green, blue):
        """Sets the brightness of the red, green, and blue light.

        Arguments:
            red (:ref:`brightness`): Brightness of the red light.
            green (:ref:`brightness`): Brightness of the green light.
            blue (:ref:`brightness`): Brightness of the blue light.
        """
        pass


class LightArray:
    """Control an array of single-color lights."""

    def __init__(self, lights):
        """Initializes the light array.

        Arguments:
            lights (int): Number of lights
        """
        pass

    def on(self, brightness=100):
        """Turns on all the lights at the specified brightness.

        Arguments:
            brightness (:ref:`brightness`):
                Brightness of the lights.
        """
        pass

    def off(self):
        """Turns off all the lights."""
        pass

    def array(self, *brightnesses):
        """array(first_light, ..., last_light)

        Sets the brightness of each light individually.

        Arguments:
            brightness (:ref:`brightness`, ..., :ref:`brightness`):
                Brightness of each light.
        """
        pass


class LightGrid:
    """Control a rectangular grid of single-color lights."""

    def __init__(self, rows, columns):
        """Initializes the light grid.

        Arguments:
            rows (int): Number of rows in the grid
            columns (int): Number of columns in the grid
        """
        pass

    def image(self, matrix, clear=True):
        """Shows an image made up of pixels of a given brightness.

        Arguments:
            matrix (2D Array): Matrix of intensities (:ref:`brightness`).
            clear (bool): Whether to turn off all the lights before showing
                the new image. If you choose ``False``,
                the given matrix is added to one already shown.
        """
        pass

    def pixel(self, row, column, brightness):
        """Turns on a pixel at the specified brightness.

        Arguments:
            row (int): Vertical grid index, starting at 0 from the top.
            column (int): Horizontal grid index, starting at 0 from the left.
            brightness (:ref:`brightness`): Brightness of the pixel.
        """
        pass

    def on(self, brightness=100):
        """Turns on all the pixels at the specified brightness.

        Arguments:
            brightness (:ref:`brightness`):
                Brightness of the lights.
        """
        pass

    def off(self):
        """Turns off all the pixels."""
        pass

    def number(self, number):
        """Displays a number on the light grid.

        Arguments:
            number (int): The number to be displayed.
        """
        pass

    def char(self, character):
        """Displays a character or symbol on the light grid. This may
        be any letter (``a``--``z``), capital letter (``A``--``Z``) or one of
        the following symbols: ``!"#$%&'()*+,-./:;<=>?@[\\]^_`{|}``.

        Arguments:
            character (str): The character or symbol to be displayed.
        """
        pass

    def text(self, text, pause=500):
        """Displays a text string, one character at a time, with a pause
        between each character. After the last character is shown, the light
        grid turns off.

        Arguments:
            character (str): The character or symbol to be displayed.
            time (:ref:`time`): How long to show a character before showing
                                the next one.
        """
        pass


class KeyPad:
    """Get status of buttons on a keypad layout."""

    def pressed(self):
        """Checks which buttons are currently pressed.

        :returns: List of pressed buttons.
        :rtype: List of :class:`Button <.parameters.Button>`

        """
        pass


class Battery:
    """Get the status of a battery."""

    def voltage(self):
        """Gets the voltage of the battery.

        Returns:
            :ref:`voltage`: Battery voltage.
        """
        pass

    def current(self):
        """Gets the current supplied by the battery.

        Returns:
            :ref:`current`: Battery current.

        """
        pass


class Accelerometer:
    """Get measurements from an accelerometer."""

    def neutral(self, top, front):
        """Configures the neutral orientation of the device or hub. You do this
        by specifying how it is mounted on your design, in terms of the
        :ref:`robot reference frame <robotframe>`.

        In this given neutral orientation, the tilt and heading will then be
        zero.

        Arguments:
            top (Axis): Which direction the top of the device faces in the
                        neutral orientation. For example, you can
                        choose ``top=-Axis.Z`` if you mounted it such that the
                        neutral orientation is upside down.
            front (Axis): Which direction the front of the device faces in the
                        neutral orientation.
        """
        pass

    def acceleration(self, axis=Axis.ALL):
        """acceleration(axis=Axis.ALL)

        Gets the acceleration of the device along a given axis in the
        :ref:`robot reference frame <robotframe>`.

        Arguments:
            axis (Axis): Axis along which the acceleration is
                         measured.
        Returns:
            :ref:`linacceleration`. Returns a :ref:`scalar` of the acceleration
            along the specified axis.
            If you choose ``axis=Axis.ALL``, you get a :ref:`vector` with the
            accelerations along all three axes (x, y, z).

        """
        pass

    def tilt(self):
        """Gets the pitch and roll angles relative to the neutral, horizontal
        orientation.

        The order of rotation is pitch-then-roll. This is equivalent to a
        positive rotation along the x-axis and then a positive rotation
        along the y-axis.

        Returns:
            (:ref:`angle`, :ref:`angle`): Pitch and roll angles.

        """
        pass

    def tapped(self):
        """Checks if the device or hub was tapped.

        Returns:
            bool:
                ``True`` if tapped since this method was last called. ``False``
                otherwise.

        """
        # def tapped(self, axis=Axis.ALL, bidirectional=True, tolerance=45):
        pass

    def shaken(self):
        """Checks if the device or hub was shaken.

        Returns:
            bool:
                ``True`` if shaken since this method was last called. ``False``
                otherwise.

        """
        # def shaken(self, axis=Axis.ALL, bidirectional=True, tolerance=45):
        pass

    def up(self):
        """Checks which side of the device or hub currently faces upward.

        :returns:
            ``Side.TOP``, ``Side.BOTTOM``, ``Side.LEFT``, ``Side.RIGHT``,
            ``Side.FRONT`` or ``Side.BACK``.
        :rtype: :class:`Side <.parameters.Side>`

        """
        pass


class IMU(Accelerometer):

    def heading(self):
        """Gets the heading angle relative to the starting orientation. It is a
        a positive rotation around the :ref:`z-axis in the robot
        frame <robotframe>`, prior to applying any tilt rotation.

        For a vehicle viewed from the top, this means that
        a positive heading value corresponds to a counterclockwise rotation.

        Returns:
            :ref:`angle`: Heading angle relative to starting orientation.

        """
        pass

    def reset_heading(self, angle):
        """Resets the accumulated heading angle of the robot.

        Arguments:
            angle (:ref:`angle`): Value to which the heading should be reset.
        """
        pass

    def gyro(self, axis=Axis.ALL):
        """gyro(axis=Axis.ALL)

        Measures the angular velocity of the device along a given axis in the
        :ref:`robot reference frame <robotframe>`.

        Arguments:
            axis (Axis): Axis along which the angular velocity is
                         measured.
        Returns:
            :ref:`speed`. Returns a :ref:`scalar` of the angular velocity
            along the specified axis.
            If you choose ``axis=Axis.ALL``, you get a :ref:`vector` with the
            angular velocities along all three axes (x, y, z).

        """
        pass



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""LEGO® Powered Up motor, sensors, and lights."""

from ._common import (KeyPad as _KeyPad, Accelerometer as _Accelerometer,
                      ColorLight as _ColorLight, Motor as _Motor)


class Motor(_Motor):
    """Generic class to control motors with built-in rotation sensors."""

    def reset_angle(self, angle=None):
        """Sets the accumulated rotation angle of the motor to a desired value.

        If you don't specify an angle, the absolute angle
        will be used if your motor supports it.

        Arguments:
            angle (:ref:`angle`): Value to which the angle should be reset.
        """
        pass


class RemoteControl:
    """LEGO® Powered Up Bluetooth Remote Control/Handset."""

    light = _ColorLight()
    buttons = _KeyPad()

    def __init__(self, device=None, timeout=10000):
        """Connect the handset to the hub.

        Arguments:
            device (str): Bluetooth address of the handset (? TODO ?).
                 If you do not specify a device identifier, the hub will
                 attempt to pair with any handset that is currently in
                 advertising mode.
            timeout (:ref:`time`): The amount of time before giving up
                 searching.
        """
        pass


class TiltSensor(_Accelerometer):
    """LEGO® Powered Up Tilt Sensor."""

    def __init__(self, port):
        """TiltSensor(port)

        Arguments:
            port (Port): Port to which the sensor is connected.
        """
        pass


class ColorDistanceSensor:
    """LEGO® Powered Up Color and Distance Sensor."""

    light = _ColorLight()

    def __init__(self, port):
        """ColorDistanceSensor(port)

        Arguments:
            port (Port): Port to which the sensor is connected.
        """
        pass

    def color(self):
        """Measures the color of a surface.

        :returns:
            ``Color.BLACK``, ``Color.BLUE``, ``Color.GREEN``, ``Color.YELLOW``,
            ``Color.RED``, ``Color.WHITE``, or ``None``.
        :rtype: :class:`Color <.parameters.Color>`, or ``None`` if no color is
                detected.
        """
        pass

    def ambient(self):
        """Measures the ambient light intensity.

        Returns:
            :ref:`percentage`: Ambient light intensity, ranging from 0 (dark)
            to 100 (bright).
        """
        pass

    def reflection(self):
        """Measures the reflection of a surface using a red light.

        Returns:
            :ref:`percentage`: Reflection, ranging from 0.0 (no reflection) to
            100.0 (high reflection).
        """
        pass

    def rgb(self):
        """Measures the reflection of a surface using a red, green, and then a
        blue light.

        :returns: Tuple of reflections for red, green, and blue light, each
                  ranging from 0.0 (no reflection) to 100.0 (high reflection).
        :rtype: (:ref:`percentage`, :ref:`percentage`, :ref:`percentage`)
        """
        pass

    def distance(self):
        """Measures the relative distance between the sensor and an object
        using infrared light.

        Returns:
            :ref:`relativedistance`: Relative distance ranging from 0 (closest)
            to 100 (farthest).
        """
        pass

    def remote(self, channel, button_1=None, button_2=None):
        """Makes the sensor act like a Power Functions 1.0 IR remote.

        Choose a channel and up to two buttons to "press". The infrared
        receiver behaves just as if responding to the real remote.

        The sensor keeps sending the signal (as if you keep pressing the
        buttons). It keeps going until you call this method again with
        different buttons or no buttons at all.

        Arguments:
            channel (int): Channel number of the remote.
            button_1 (Button): Button of the Power Functions 1.0 IR remote.
                               Choose ``None`` if you don't want to press
                               any button.
            button_2 (Button): Button of the Power Functions 1.0 IR remote.
                               Choose ``None`` if you don't want to press
                               any button.
        """



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from typing import Optional, Tuple, overload
from ._common import Control, Motor

class DriveBase:
    distance_control: Control
    heading_control: Control
    def __init__(
        self,
        left_motor: Motor,
        right_motor: Motor,
        wheel_diameter: int,
        axle_track: int,
    ): ...
    def drive(self, drive_speed: int, turn_rate: int) -> None: ...
    def stop(self) -> None: ...
    def distance(self) -> int: ...
    def angle(self) -> int: ...
    def state(self) -> Tuple[int, int, int, int]: ...
    def reset(self) -> None: ...
    @overload
    def settings(self) -> Tuple[int, int, int, int]: ...
    @overload
    def settings(
        self,
        straight_speed: Optional[int],
        straight_acceleration: Optional[int],
        turn_rate: Optional[int],
        turn_acceleration: Optional[int],
    ) -> None: ...
    def straight(self, distance: int) -> None: ...
    def turn(self, angle: int) -> None: ...



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""LEGO® Programmable Hubs."""
from ._common import (Speaker as _Speaker, Battery as _Battery,
                      ColorLight as _ColorLight, KeyPad as _KeyPad)
from .media.ev3dev import Image as _Image


class EV3Brick:
    """LEGO® MINDSTORMS® EV3 Brick."""
    screen = _Image('_screen_')
    speaker = _Speaker()
    battery = _Battery()
    light = _ColorLight()
    buttons = _KeyPad()


class MoveHub:
    """LEGO® Powered Up Move Hub."""
    battery = _Battery()
    light = _ColorLight()


class CityHub:
    """LEGO® Powered Up City Hub."""
    battery = _Battery()
    light = _ColorLight()



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""Images and Sounds for Pybricks on ev3dev."""

from ..parameters import Color as _Color


class Image:
    """Object representing a graphics image. This can either be an in-memory
    copy of an image or the image displayed on a screen."""

    # Documentation note: This class is also treated as the `screen` object
    # on EV3 so we use |this image| when it would make sense to say "the screen"
    # in that context and it is automatically replaced when the documentation
    # is generated.

    def __init__(self, source, sub=False):
        """
        Arguments:
            source (str or Image):
                The source of the image.

                If ``source`` is a string, then the image will be loaded from
                the file path given by the string. Only ``.png`` files are
                supported. As a special case, if the string is ``_screen_``,
                the image will be configured to draw directly on the screen.

                If an :class:`Image` is given, the new object will contain a
                copy of the ``source`` image object.

            sub (bool):
                If ``sub`` is ``True``, then the image object will act as a
                sub-image of the ``source`` image (this only works if the type
                of ``source`` is :class:`Image` and not when it is a ``str``).

                Additional keyword arguments ``x1``, ``y1``, ``x2``, ``y2`` are
                needed when ``sub=True``. These specify the top-left and
                bottom-right coordinates in the ``source`` image that will be
                used as the bounds for the sub-image.
        """
        pass

    @property
    def width(self):
        """Gets the width of |this image| in pixels."""
        return 0

    @property
    def height(self):
        """Gets the height of |this image| in pixels."""
        return 0

    def clear(self):
        """Clears |this image|. All pixels on |this image| will be set to
        :attr:`Color.WHITE <pybricks.parameters.Color.WHITE>`.
        """
        pass

    def draw_pixel(self, x, y, color=_Color.BLACK):
        """Draws a single pixel on |this image|.

        Arguments:
            x (int): The x coordinate of the pixel.
            y (int): The y coordinate of the pixel.
            color (Color): The color of the pixel.
        """
        pass

    def draw_line(self, x1, y1, x2, y2, width=1, color=_Color.BLACK):
        """Draws a line on |this image|.

        Arguments:
            x1 (int): The x coordinate of the starting point of the line.
            y1 (int): The y coordinate of the starting point of the line.
            x2 (int): The x coordinate of the ending point of the line.
            y2 (int): The y coordinate of the ending point of the line.
            width (int): The width of the line in pixels.
            color (Color): The color of the line.
        """
        pass

    def draw_box(self, x1, y1, x2, y2, r=0, fill=False, color=_Color.BLACK):
        """Draws a box on |this image|.

        Arguments:
            x1 (int): The x coordinate of the left side of the box.
            y1 (int): The y coordinate of the top of the box.
            x2 (int): The x coordinate of the right side of the box.
            y2 (int): The y coordinate of the bottom of the box.
            r (int): The radius of the corners of the box.
            fill (bool): If ``True``, the box will be filled with ``color``,
                otherwise only the outline of the box will be drawn.
            color (Color): The color of the box.
        """
        pass

    def draw_circle(self, x, y, r, fill=False, color=_Color.BLACK):
        """Draws a circle on |this image|.

        Arguments:
            x (int): The x coordinate of the center of the circle.
            y (int): The y coordinate of the center of the circle.
            r (int): The radius of the circle.
            fill (bool): If ``True``, the circle will be filled with
                ``color``, otherwise only the circumference will be drawn.
            color (Color): The color of the circle.
        """
        pass

    def draw_image(self, x, y, source, transparent=None):
        """Draws the ``source`` image on |this image|.

        Arguments:
            x (int):
                The x-axis value where the left side of the image will start.
            y (int):
                The y-axis value where the top of the image will start.
            source (Image or str):
                The source :class:`Image`. If the argument is a string, then
                the ``source`` image is loaded from file.
            transparent (Color):
                The color of ``image`` to treat as transparent or ``None`` for
                no transparency.
        """

    def load_image(self, source):
        """Clears this image, then draws the ``source`` image centered in
        |this image|.

        Arguments:
            source (Image or str):
                The source :class:`Image`. If the argument is a string, then
                the ``source`` image is loaded from file.
        """

    def draw_text(self, x, y, text, text_color=_Color.BLACK, background_color=None):
        """Draws text on |this image|.

        The most recent font set using :meth:`set_font` will be used or
        :data:`Font.DEFAULT` if no font has been set yet.

        Arguments:
            x (int):
                The x-axis value where the left side of the text will start.
            y (int):
                The y-axis value where the top of the text will start.
            text (str):
                The text to draw.
            text_color (Color):
                The color used for drawing the text.
            background_color (Color):
                The color used to fill the rectangle behind the text or ``None``
                for transparent background.
        """
        pass

    def print(self, *args, sep=' ', end='\n'):
        """Prints a line of text on |this image|.

        This method works like the builtin ``print()`` function, but it writes
        on |this image| instead.

        You can set the font using :meth:`set_font`. If no font has been set,
        :data:`Font.DEFAULT` will be used. The text is always printed used
        black text with a white background.

        Unlike the builtin ``print()``, the text does not wrap if it is too
        wide to fit on |this image|. It just gets cut off. But if the text would
        go off of the bottom of |this image|, the entire image is scrolled up and
        the text is printed in the new blank area at the bottom of |this image|.

        Arguments:
            * (object):
                Zero or more objects to print.
            sep (str):
                Separator that will be placed between each object that is
                printed.
            end (str):
                End of line that will be printed after the last object.
        """
        pass

    def set_font(self, font):
        """Sets the font used for writing on |this image|.

        The font is used for both :meth:`draw_text` and :meth:`print`.

        Arguments:
            font (:class:`Font`):
                The font to use.
        """
        pass

    @staticmethod
    def empty(width=178, height=128):
        """empty(width=<screen width>, height=<screen height>)

        Creates a new empty :class:`Image` object.

        Arguments:
            width (int):
                The width of the image in pixels.
            height (int):
                The height of the image in pixels.

        Returns:
            Image:
                A new image with all pixels set to :attr:`Color.WHITE
                <pybricks.parameters.Color.WHITE>`.

        Raises:
            TypeError:
                ``width`` or ``height`` is not a number.
            ValueError:
                ``width`` or ``height`` is less than 1.
            RuntimeError:
                There was a problem allocating a new image.
        """

    def save(self, filename):
        """Saves |this image| as a ``.png`` file.

        Arguments:
            filename (str):
                The path to the file to be saved.

        Raises:
            TypeError:
                ``filename`` is not a string.
            OSError:
                There was a problem saving the file.
        """


class Font:
    """Object that represents a font for writing text."""

    DEFAULT = None  # assigned later since we can't use Font() here
    """The default font."""

    def __init__(self, family=None, size=12, bold=False, monospace=False,
                 lang=None, script=None):
        """The font object will be a font that is the "best" match based on the
        parameters given and available fonts installed.

        Arguments:
            family (str):
                The preferred font family or ``None`` to use the default value.
            size (int):
                The preferred font size. Most fonts have sizes between 6 and 24.
                This is the "point" size and not the same as :attr:`height`.
            bold (bool):
                When ``True``, prefer bold fonts.
            monospace (bool):
                When ``True`` prefer monospaced fonts. This is useful for
                aligning multiple rows of text.
            lang (str):
                A language code, such as ``'en'`` or ``'zh-cn'`` or ``None`` to
                use the default language. [#font_lang]_
            script (str):
                A unicode script identifier such as ``'Runr'`` or ``None``.

        .. [#font_lang]
            .. toggle-header::
                :header: Language codes

                Note: Languages depend on installed fonts. Additional language
                codes are possible and some listed language codes may not have
                a satisfactory font.

                - ``'aa'``: Afar
                - ``'af'``: Afrikaans
                - ``'an'``: Aragonese
                - ``'av'``: Avaric
                - ``'ay'``: Aymara
                - ``'az-az'``: Azerbaijani
                - ``'be'``: Belarusian
                - ``'bg'``: Bulgarian
                - ``'bi'``: Bislama
                - ``'bm'``: Bambara
                - ``'br'``: Breton
                - ``'bs'``: Bosnian
                - ``'bua'``: Buriat
                - ``'ca'``: Catalan
                - ``'ce'``: Chechen
                - ``'ch'``: Chamorro
                - ``'co'``: Corsican
                - ``'crh'``: Crimean
                - ``'cs'``: Czech
                - ``'csb'``: Kashubian
                - ``'cy'``: Welsh
                - ``'da'``: Danish
                - ``'de'``: German
                - ``'ee'``: Ewe
                - ``'el'``: Greek
                - ``'en'``: English
                - ``'eo'``: Esperanto
                - ``'es'``: Spanish
                - ``'et'``: Estonian
                - ``'eu'``: Basque
                - ``'ff'``: Fulah
                - ``'fi'``: Finnish
                - ``'fil'``: Filipino
                - ``'fj'``: Fijian
                - ``'fo'``: Faroese
                - ``'fr'``: French
                - ``'fur'``: Friulian
                - ``'fy'``: Western Frisian
                - ``'ga'``: Irish
                - ``'gd'``: Gaelic
                - ``'gl'``: Galician
                - ``'gv'``: Manx
                - ``'ha'``: Hausa
                - ``'haw'``: Hawaiian
                - ``'he'``: Hebrew
                - ``'ho'``: Hiri Motu
                - ``'hr'``: Croatian
                - ``'hsb'``: Upper Sorbian
                - ``'ht'``: Haitian
                - ``'hu'``: Hungarian
                - ``'ia'``: Interlingua
                - ``'id'``: Indonesian
                - ``'ie'``: Interlingue
                - ``'ik'``: Inupiaq
                - ``'io'``: Ido
                - ``'is'``: Icelandic
                - ``'it'``: Italian
                - ``'ja'``: Japanese
                - ``'jv'``: Javanese
                - ``'ki'``: Kikuyu
                - ``'kj'``: Kuanyama
                - ``'kl'``: Kalaallisut
                - ``'ko'``: Korean
                - ``'ku-tr'``: Kurdish
                - ``'kum'``: Kumyk
                - ``'kw'``: Cornish
                - ``'kwm'``: Kwambi
                - ``'la'``: Latin
                - ``'lb'``: Luxembourgish
                - ``'lez'``: Lezghian
                - ``'lg'``: Ganda
                - ``'li'``: Limburgan
                - ``'ln'``: Lingala
                - ``'lt'``: Lithuanian
                - ``'lv'``: Latvian
                - ``'mg'``: Malagasy
                - ``'mh'``: Marshallese
                - ``'mi'``: Maori
                - ``'mk'``: Macedonian
                - ``'mn-mn'``: Mongolian
                - ``'mo'``: Moldavian
                - ``'ms'``: Malay
                - ``'mt'``: Maltese
                - ``'na'``: Nauru
                - ``'nb'``: Norwegian Bokmål
                - ``'nds'``: Low German
                - ``'ng'``: Ndonga
                - ``'nl'``: Dutch
                - ``'nn'``: Norwegian Nynorsk
                - ``'no'``: Norwegian
                - ``'nr'``: South Ndebele
                - ``'nso'``: Northern Sotho
                - ``'nv'``: Navajo
                - ``'ny'``: Chichewa
                - ``'oc'``: Occitan
                - ``'om'``: Oromo
                - ``'os'``: Ossetian
                - ``'pap-an'``: Papiamento, Netherlands Antilles
                - ``'pap-aw'``: Papiamento, Aruba
                - ``'pl'``: Polish
                - ``'pt'``: Portuguese
                - ``'qu'``: Quechua
                - ``'quz'``: Cusco Quechua
                - ``'rm'``: Romansh
                - ``'rn'``: Rundi
                - ``'ro'``: Romanian
                - ``'ru'``: Russian
                - ``'rw'``: Kinyarwanda
                - ``'sc'``: Sardinian
                - ``'sco'``: Scots
                - ``'se'``: Northern Sami
                - ``'sel'``: Selkup
                - ``'sg'``: Sango
                - ``'sk'``: Slovak
                - ``'sl'``: Slovenian
                - ``'sm'``: Samoan
                - ``'sma'``: Southern Sami
                - ``'smj'``: Lule Sami
                - ``'smn'``: Inari Sami
                - ``'sms'``: Skolt Sami
                - ``'sn'``: Shona
                - ``'so'``: Somali
                - ``'sq'``: Albanian
                - ``'sr'``: Serbian
                - ``'ss'``: Swati
                - ``'st'``: Southern Sotho
                - ``'su'``: Sundanese
                - ``'sv'``: Swedish
                - ``'sw'``: Swahili
                - ``'tk'``: Turkmen
                - ``'tl'``: Tagalog
                - ``'tn'``: Tswana
                - ``'to'``: Tonga
                - ``'tr'``: Turkish
                - ``'ts'``: Tsonga
                - ``'ty'``: Tahitian
                - ``'uk'``: Ukrainian
                - ``'uz'``: Uzbek
                - ``'vo'``: Volapük
                - ``'vot'``: Votic
                - ``'wa'``: Walloon
                - ``'wen'``: Sorbian
                - ``'wo'``: Wolof
                - ``'xh'``: Xhosa
                - ``'yap'``: Yapese
                - ``'yi'``: Yiddish
                - ``'za'``: Zhuang
                - ``'zh-cn'``: Chinese, China
                - ``'zh-sg'``: Chinese, Singapore
                - ``'zh-tw'``: Chinese, Taiwan
                - ``'zu'``: Zulu
        """

    @property
    def family(self):
        """Gets the family name of the font."""
        return 'Lucida'

    @property
    def style(self):
        """Gets a string describing the font style.

        Can be "Regular" or "Bold".
        """
        return 'Regular'

    @property
    def width(self):
        """Gets the width of the widest character of the font."""
        return 0

    @property
    def height(self):
        """Gets the height of the font."""
        return 0

    def text_width(self, text):
        """Gets the width of the text when the text is drawn using this font.

        Arguments:
            text (str):
                The text.

        Returns:
            int:
                The width in pixels.
        """
        return 0

    def text_height(self, text):
        """Gets the height of the text when the text is drawn using this font.

        Arguments:
            text (str):
                The text.

        Returns:
            int:
                The height in pixels.
        """
        return 0


Font.DEFAULT = Font('Lucida', 12)


class SoundFile:
    """Paths to standard EV3 sounds."""

    _BASE_PATH = '/usr/share/sounds/ev3dev/'
    SHOUTING = _BASE_PATH + 'expressions/shouting.wav'
    CHEERING = _BASE_PATH + 'expressions/cheering.wav'
    CRYING = _BASE_PATH + 'expressions/crying.wav'
    OUCH = _BASE_PATH + 'expressions/ouch.wav'
    LAUGHING_2 = _BASE_PATH + 'expressions/laughing_2.wav'
    SNEEZING = _BASE_PATH + 'expressions/sneezing.wav'
    SMACK = _BASE_PATH + 'expressions/smack.wav'
    BOING = _BASE_PATH + 'expressions/boing.wav'
    BOO = _BASE_PATH + 'expressions/boo.wav'
    UH_OH = _BASE_PATH + 'expressions/uh-oh.wav'
    SNORING = _BASE_PATH + 'expressions/snoring.wav'
    KUNG_FU = _BASE_PATH + 'expressions/kung_fu.wav'
    FANFARE = _BASE_PATH + 'expressions/fanfare.wav'
    CRUNCHING = _BASE_PATH + 'expressions/crunching.wav'
    MAGIC_WAND = _BASE_PATH + 'expressions/magic_wand.wav'
    LAUGHING_1 = _BASE_PATH + 'expressions/laughing_1.wav'
    LEFT = _BASE_PATH + 'information/left.wav'
    BACKWARDS = _BASE_PATH + 'information/backwards.wav'
    RIGHT = _BASE_PATH + 'information/right.wav'
    OBJECT = _BASE_PATH + 'information/object.wav'
    COLOR = _BASE_PATH + 'information/color.wav'
    FLASHING = _BASE_PATH + 'information/flashing.wav'
    ERROR = _BASE_PATH + 'information/error.wav'
    ERROR_ALARM = _BASE_PATH + 'information/error_alarm.wav'
    DOWN = _BASE_PATH + 'information/down.wav'
    FORWARD = _BASE_PATH + 'information/forward.wav'
    ACTIVATE = _BASE_PATH + 'information/activate.wav'
    SEARCHING = _BASE_PATH + 'information/searching.wav'
    TOUCH = _BASE_PATH + 'information/touch.wav'
    UP = _BASE_PATH + 'information/up.wav'
    ANALYZE = _BASE_PATH + 'information/analyze.wav'
    STOP = _BASE_PATH + 'information/stop.wav'
    DETECTED = _BASE_PATH + 'information/detected.wav'
    TURN = _BASE_PATH + 'information/turn.wav'
    START = _BASE_PATH + 'information/start.wav'
    MORNING = _BASE_PATH + 'communication/morning.wav'
    EV3 = _BASE_PATH + 'communication/ev3.wav'
    GO = _BASE_PATH + 'communication/go.wav'
    GOOD_JOB = _BASE_PATH + 'communication/good_job.wav'
    OKEY_DOKEY = _BASE_PATH + 'communication/okey-dokey.wav'
    GOOD = _BASE_PATH + 'communication/good.wav'
    NO = _BASE_PATH + 'communication/no.wav'
    THANK_YOU = _BASE_PATH + 'communication/thank_you.wav'
    YES = _BASE_PATH + 'communication/yes.wav'
    GAME_OVER = _BASE_PATH + 'communication/game_over.wav'
    OKAY = _BASE_PATH + 'communication/okay.wav'
    SORRY = _BASE_PATH + 'communication/sorry.wav'
    BRAVO = _BASE_PATH + 'communication/bravo.wav'
    GOODBYE = _BASE_PATH + 'communication/goodbye.wav'
    HI = _BASE_PATH + 'communication/hi.wav'
    HELLO = _BASE_PATH + 'communication/hello.wav'
    MINDSTORMS = _BASE_PATH + 'communication/mindstorms.wav'
    LEGO = _BASE_PATH + 'communication/lego.wav'
    FANTASTIC = _BASE_PATH + 'communication/fantastic.wav'
    SPEED_IDLE = _BASE_PATH + 'movements/speed_idle.wav'
    SPEED_DOWN = _BASE_PATH + 'movements/speed_down.wav'
    SPEED_UP = _BASE_PATH + 'movements/speed_up.wav'
    BROWN = _BASE_PATH + 'colors/brown.wav'
    GREEN = _BASE_PATH + 'colors/green.wav'
    BLACK = _BASE_PATH + 'colors/black.wav'
    WHITE = _BASE_PATH + 'colors/white.wav'
    RED = _BASE_PATH + 'colors/red.wav'
    BLUE = _BASE_PATH + 'colors/blue.wav'
    YELLOW = _BASE_PATH + 'colors/yellow.wav'
    TICK_TACK = _BASE_PATH + 'mechanical/tick_tack.wav'
    HORN_1 = _BASE_PATH + 'mechanical/horn_1.wav'
    BACKING_ALERT = _BASE_PATH + 'mechanical/backing_alert.wav'
    MOTOR_IDLE = _BASE_PATH + 'mechanical/motor_idle.wav'
    AIR_RELEASE = _BASE_PATH + 'mechanical/air_release.wav'
    AIRBRAKE = _BASE_PATH + 'mechanical/airbrake.wav'
    RATCHET = _BASE_PATH + 'mechanical/ratchet.wav'
    MOTOR_STOP = _BASE_PATH + 'mechanical/motor_stop.wav'
    HORN_2 = _BASE_PATH + 'mechanical/horn_2.wav'
    LASER = _BASE_PATH + 'mechanical/laser.wav'
    SONAR = _BASE_PATH + 'mechanical/sonar.wav'
    MOTOR_START = _BASE_PATH + 'mechanical/motor_start.wav'
    INSECT_BUZZ_2 = _BASE_PATH + 'animals/insect_buzz_2.wav'
    ELEPHANT_CALL = _BASE_PATH + 'animals/elephant_call.wav'
    SNAKE_HISS = _BASE_PATH + 'animals/snake_hiss.wav'
    DOG_BARK_2 = _BASE_PATH + 'animals/dog_bark_2.wav'
    DOG_WHINE = _BASE_PATH + 'animals/dog_whine.wav'
    INSECT_BUZZ_1 = _BASE_PATH + 'animals/insect_buzz_1.wav'
    DOG_SNIFF = _BASE_PATH + 'animals/dog_sniff.wav'
    T_REX_ROAR = _BASE_PATH + 'animals/t-rex_roar.wav'
    INSECT_CHIRP = _BASE_PATH + 'animals/insect_chirp.wav'
    DOG_GROWL = _BASE_PATH + 'animals/dog_growl.wav'
    SNAKE_RATTLE = _BASE_PATH + 'animals/snake_rattle.wav'
    DOG_BARK_1 = _BASE_PATH + 'animals/dog_bark_1.wav'
    CAT_PURR = _BASE_PATH + 'animals/cat_purr.wav'
    EIGHT = _BASE_PATH + 'numbers/eight.wav'
    SEVEN = _BASE_PATH + 'numbers/seven.wav'
    SIX = _BASE_PATH + 'numbers/six.wav'
    FOUR = _BASE_PATH + 'numbers/four.wav'
    TEN = _BASE_PATH + 'numbers/ten.wav'
    ONE = _BASE_PATH + 'numbers/one.wav'
    TWO = _BASE_PATH + 'numbers/two.wav'
    THREE = _BASE_PATH + 'numbers/three.wav'
    ZERO = _BASE_PATH + 'numbers/zero.wav'
    FIVE = _BASE_PATH + 'numbers/five.wav'
    NINE = _BASE_PATH + 'numbers/nine.wav'
    READY = _BASE_PATH + 'system/ready.wav'
    CONFIRM = _BASE_PATH + 'system/confirm.wav'
    GENERAL_ALERT = _BASE_PATH + 'system/general_alert.wav'
    CLICK = _BASE_PATH + 'system/click.wav'
    OVERPOWER = _BASE_PATH + 'system/overpower.wav'


class ImageFile:
    """Paths to standard EV3 images."""

    _BASE_PATH = '/usr/share/images/ev3dev/mono/'
    RIGHT = _BASE_PATH + 'information/right.png'
    FORWARD = _BASE_PATH + 'information/forward.png'
    ACCEPT = _BASE_PATH + 'information/accept.png'
    QUESTION_MARK = _BASE_PATH + 'information/question_mark.png'
    STOP_1 = _BASE_PATH + 'information/stop_1.png'
    LEFT = _BASE_PATH + 'information/left.png'
    DECLINE = _BASE_PATH + 'information/decline.png'
    THUMBS_DOWN = _BASE_PATH + 'information/thumbs_down.png'
    BACKWARD = _BASE_PATH + 'information/backward.png'
    NO_GO = _BASE_PATH + 'information/no_go.png'
    WARNING = _BASE_PATH + 'information/warning.png'
    STOP_2 = _BASE_PATH + 'information/stop_2.png'
    THUMBS_UP = _BASE_PATH + 'information/thumbs_up.png'
    EV3 = _BASE_PATH + 'lego/ev3.png'
    EV3_ICON = _BASE_PATH + 'lego/ev3_icon.png'
    TARGET = _BASE_PATH + 'objects/target.png'
    BOTTOM_RIGHT = _BASE_PATH + 'eyes/bottom_right.png'
    BOTTOM_LEFT = _BASE_PATH + 'eyes/bottom_left.png'
    EVIL = _BASE_PATH + 'eyes/evil.png'
    CRAZY_2 = _BASE_PATH + 'eyes/crazy_2.png'
    KNOCKED_OUT = _BASE_PATH + 'eyes/knocked_out.png'
    PINCHED_RIGHT = _BASE_PATH + 'eyes/pinched_right.png'
    WINKING = _BASE_PATH + 'eyes/winking.png'
    DIZZY = _BASE_PATH + 'eyes/dizzy.png'
    DOWN = _BASE_PATH + 'eyes/down.png'
    TIRED_MIDDLE = _BASE_PATH + 'eyes/tired_middle.png'
    MIDDLE_RIGHT = _BASE_PATH + 'eyes/middle_right.png'
    SLEEPING = _BASE_PATH + 'eyes/sleeping.png'
    MIDDLE_LEFT = _BASE_PATH + 'eyes/middle_left.png'
    TIRED_RIGHT = _BASE_PATH + 'eyes/tired_right.png'
    PINCHED_LEFT = _BASE_PATH + 'eyes/pinched_left.png'
    PINCHED_MIDDLE = _BASE_PATH + 'eyes/pinched_middle.png'
    CRAZY_1 = _BASE_PATH + 'eyes/crazy_1.png'
    NEUTRAL = _BASE_PATH + 'eyes/neutral.png'
    AWAKE = _BASE_PATH + 'eyes/awake.png'
    UP = _BASE_PATH + 'eyes/up.png'
    TIRED_LEFT = _BASE_PATH + 'eyes/tired_left.png'
    ANGRY = _BASE_PATH + 'eyes/angry.png'



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from abc import abstractmethod
from typing import Callable, Generic, Optional, TypeVar

T = TypeVar("T")

class Connection:
    @abstractmethod
    def read_from_mailbox(self, name: str) -> bytes: ...
    @abstractmethod
    def send_to_mailbox(self, name: str, data: bytes) -> None: ...
    @abstractmethod
    def wait_for_mailbox_update(self, name: str) -> None: ...

class Mailbox(Generic[T]):
    def __init__(
        self,
        name: str,
        connection: Connection,
        encode: Optional[Callable[[T], bytes]] = None,
        decode: Optional[Callable[[bytes], T]] = None,
    ): ...
    def read(self) -> T: ...
    def send(self, value: T, brick: Optional[str] = None) -> None: ...
    def wait(self) -> None: ...
    def wait_new(self) -> T: ...

class LogicMailbox(Mailbox[bool]):
    def __init__(self, name, connection: Connection): ...

class NumericMailbox(Mailbox[float]):
    def __init__(self, name, connection: Connection): ...

class TextMailbox(Mailbox[str]):
    def __init__(self, name, connection: Connection): ...

class BluetoothMailboxServer(Connection):
    def __enter__(self) -> BluetoothMailboxServer: ...
    def __exit__(self, type, value, traceback) -> None: ...
    def wait_for_connection(self, count: int = 1) -> None: ...
    def close(self) -> None: ...

class BluetoothMailboxClient(Connection):
    def __enter__(self) -> BluetoothMailboxClient: ...
    def __exit__(self, type, value, traceback) -> None: ...
    def connect(self, brick: str) -> None: ...
    def close(self) -> None: ...



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from typing import List, Optional, Tuple

from .parameters import Color, Direction, Port, Button
from ._common import Motor  # noqa E402

class TouchSensor:
    def __init__(self, port: Port): ...
    def pressed(self) -> bool: ...

class ColorSensor:
    def __init__(self, port: Port): ...
    def color(self) -> Optional[Color]: ...
    def ambient(self) -> int: ...
    def reflection(self) -> int: ...
    def rgb(self) -> Tuple[int, int, int]: ...

class InfraredSensor:
    def __init__(self, port: Port): ...
    def distance(self) -> int: ...
    def beacon(self, channel: int) -> Tuple[Optional[int], Optional[int]]: ...
    def buttons(self, channel: int) -> List[Button]: ...
    def keypad(self) -> List[Button]: ...

class GyroSensor:
    def __init__(
        self, port: Port, positive_direction: Direction = Direction.CLOCKWISE
    ): ...
    def speed(self) -> int: ...
    def angle(self) -> int: ...
    def reset_angle(self, angle: int) -> None: ...

class UltrasonicSensor:
    def __init__(self, port: Port): ...
    def distance(self, silent: bool = False) -> int: ...
    def presence(self) -> bool: ...



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""Use LEGO® MINDSTORMS® NXT motors and sensors with the EV3 brick."""


from .iodevices import AnalogSensor as _AnalogSensor
from ._common import ColorLight as _ColorLight


class TouchSensor:
    """LEGO® MINDSTORMS® NXT Touch Sensor."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.
        """
        pass

    def pressed(self):
        """Checks if the sensor is pressed.

        Returns:
            bool: ``True`` if the sensor is pressed, ``False`` if it is
            not pressed.

        """
        pass


class LightSensor:
    """LEGO® MINDSTORMS® NXT Color Sensor."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def ambient(self):
        """Measures the ambient light intensity.

        Returns:
            :ref:`percentage`: Ambient light intensity, ranging from 0 (dark)
            to 100 (bright).
        """
        pass

    def reflection(self):
        """Measures the reflection of a surface using a red light.

        Returns:
            :ref:`percentage`: Reflection, ranging from 0 (no reflection) to
            100 (high reflection).

        """
        pass


class ColorSensor:
    """LEGO® MINDSTORMS® NXT Color Sensor."""

    light = _ColorLight()

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def color(self):
        """Measures the color of a surface.

        :returns:
            ``Color.BLACK``, ``Color.BLUE``, ``Color.GREEN``, ``Color.YELLOW``,
            ``Color.RED``, ``Color.WHITE`` or ``None``.
        :rtype: :class:`Color <.parameters.Color>`, or ``None`` if no color is
                detected.

        """
        pass

    def ambient(self):
        """Measures the ambient light intensity.

        Returns:
            :ref:`percentage`: Ambient light intensity, ranging from 0 (dark)
            to 100 (bright).
        """
        pass

    def reflection(self):
        """Measures the reflection of a surface.

        Returns:
            :ref:`percentage`: Reflection, ranging from 0 (no reflection) to
            100 (high reflection).

        """
        pass

    def rgb(self):
        """Measures the reflection of a surface using a red, green, and then a
        blue light.

        :returns: Tuple of reflections for red, green, and blue light, each
                  ranging from 0.0 (no reflection) to 100.0 (high reflection).
        :rtype: (:ref:`percentage`, :ref:`percentage`, :ref:`percentage`)
        """
        pass


class UltrasonicSensor:
    """LEGO® MINDSTORMS® NXT Ultrasonic Sensor."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def distance(self):
        """Measures the distance between the sensor and an object using
        ultrasonic sound waves.

        Returns:
            :ref:`distance`: Distance.

        """
        pass


class SoundSensor:
    """LEGO® MINDSTORMS® NXT Sound Sensor."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def intensity(self, audible_only=True):
        """Measures the ambient sound intensity (loudness).

        Arguments:
            audible_only (bool): Detect only audible sounds. This tries to
                filter out frequencies that cannot be heard by the
                human ear.

        Returns:
            :ref:`percentage`: Sound intensity.

        """
        pass


class TemperatureSensor:
    """LEGO® MINDSTORMS® NXT Temperature Sensor."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def temperature(self):
        """Measures the temperature.

        Returns:
            :ref:`temperature`: Measured temperature.

        """
        pass


class EnergyMeter:
    """LEGO® MINDSTORMS® Education NXT Energy Meter."""

    def __init__(self, port):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def storage(self):
        """Gets the total available energy stored in the battery.

        Returns:
            :ref:`energy`: Remaining stored energy.

        """
        pass

    def input(self):
        """Measures the electrical signals at the input (bottom) side
        of the energy meter. It measures the voltage applied to it and the
        current passing through it. The product of these two values is power.
        This power value is the rate at which the stored energy increases. This
        power is supplied by an energy source such as the provided solar panel
        or an externally driven motor.

        Returns:
            (:ref:`voltage`, :ref:`current`, :ref:`power`): Voltage, current,
            and power measured at the input port.

        """
        pass

    def output(self):
        """Measures the electrical signals at the output (top) side
        of the energy meter. It measures the voltage applied to the external
        load and the current passing to it. The product of these two values
        is power. This power value is the rate at which the stored energy
        decreases. This power is consumed by the load, such as a light or a
        motor.

        Returns:
            (:ref:`voltage`, :ref:`current`, :ref:`power`): Voltage, current,
            and power measured at the output port.

        """
        pass


class VernierAdapter(_AnalogSensor):
    """LEGO® MINDSTORMS® Education NXT/EV3 Adapter for Vernier Sensors."""

    def __init__(self, port, conversion=None):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.
            conversion (callable): Function of the format :meth:`.conversion`.
                This function is used to convert the raw analog voltage to the
                sensor-specific output value. Each Vernier Sensor has its
                own conversion function. The example given below demonstrates
                the conversion for the Surface Temperature Sensor.
        """
        pass

    def voltage(self):
        """Measures the raw analog sensor voltage.

        Returns:
            :ref:`voltage`: Analog voltage.
        """
        pass

    def conversion(self, voltage):
        """Converts the raw voltage (mV) to a sensor value.

        If you did not provide a ``conversion`` function earlier, no conversion
        will be applied.

        Arguments:
            voltage (:ref:`voltage`): Analog sensor voltage

        :returns: Converted sensor value.
        :rtype: float
        """
        pass

    def value(self):
        """Measures the sensor :meth:`.voltage` and then
        applies your :meth:`.conversion` to give you the sensor value.

        :returns: Converted sensor value.
        :rtype: float
        """
        pass



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""Common tools for timing and data logging."""


def wait(time):
    """Pauses the user program for a specified amount of time.

    Arguments:
        time (:ref:`time`): How long to wait.

    """
    pass


class StopWatch:
    """A stopwatch to measure time intervals. Similar to the stopwatch
    feature on your phone."""

    def __init__(self):
        pass

    def time(self):
        """Gets the current time of the stopwatch.

        Returns:
            :ref:`time`: Elapsed time.
        """
        pass

    def pause(self):
        """Pauses the stopwatch."""
        pass

    def resume(self):
        """Resumes the stopwatch."""
        pass

    def reset(self):
        """Resets the stopwatch time to 0.

        The run state is unaffected:

        * If it was paused, it stays paused (but now at 0).
        * If it was running, it stays running (but starting again from 0).
        """
        pass


class DataLog:
    """Create a file and log data."""

    def __init__(self, *headers, name='log', timestamp=True, extension='csv',
                 append=False):
        """

        Arguments:
            headers (`col1`, `col2`, `...`): Column headers. These are the
                names of the data columns. For example, choose ``'time'`` and
                ``'angle'``.
            name (str): Name of the file.
            timestamp (bool): Choose ``True`` to add the date and time to the
                file name. This way, your file has a unique name.
                Choose ``False`` to omit the timestamp.
            extension (str): File extension.
            append (bool): Choose ``True`` to reopen an existing data log file
                and append data to it. Choose ``False`` to clear existing
                data. If the file does not exist yet, an empty file will be
                created either way.
        """
        pass

    def log(self, *values):
        """Saves one or more values on a new line in the file.

        Arguments:
            values (object, object, `...`): One or more objects or values.
        """
        pass



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from ._common import Speaker, Battery, ColorLight, KeyPad
from .media.ev3dev import Image

class EV3Brick:
    screen: Image
    speaker: Speaker
    battery: Battery
    light: ColorLight
    buttons = KeyPad



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from typing import Any

def wait(time: int) -> None: ...

class StopWatch:
    def time(self) -> int: ...
    def pause(self) -> None: ...
    def resume(self) -> None: ...
    def reset(self) -> None: ...

class DataLog:
    def __init__(
        self,
        *headers: str,
        name: str = "log",
        timestamp: bool = True,
        extension: str = "csv",
        append: bool = False
    ): ...
    def log(self, *values: Any) -> None: ...



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""LEGO® MINDSTORMS® EV3 motors and sensors."""

from .parameters import Direction as _Direction
from ._common import Motor  # noqa E402


class TouchSensor:
    """LEGO® MINDSTORMS® EV3 Touch Sensor."""

    def __init__(self, port):
        """TouchSensor(port)

        Arguments:
            port (Port): Port to which the sensor is connected.
        """
        pass

    def pressed(self):
        """Checks if the sensor is pressed.

        Returns:
            bool: ``True`` if the sensor is pressed, ``False`` if it is
            not pressed.

        """
        pass


class ColorSensor:
    """LEGO® MINDSTORMS® EV3 Color Sensor."""

    def __init__(self, port):
        """ColorSensor(port)

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def color(self):
        """Measures the color of a surface.

        :returns:
            ``Color.BLACK``, ``Color.BLUE``, ``Color.GREEN``, ``Color.YELLOW``,
            ``Color.RED``, ``Color.WHITE``, ``Color.BROWN`` or ``None``.
        :rtype: :class:`Color <.parameters.Color>`, or ``None`` if no color is
                detected.

        """
        pass

    def ambient(self):
        """Measures the ambient light intensity.

        Returns:
            :ref:`percentage`: Ambient light intensity, ranging from 0 (dark)
            to 100 (bright).
        """
        pass

    def reflection(self):
        """Measures the reflection of a surface using a red light.

        Returns:
            :ref:`percentage`: Reflection, ranging from 0 (no reflection) to
            100 (high reflection).

        """
        pass

    def rgb(self):
        """Measures the reflection of a surface using a red, green, and then a
        blue light.

        :returns: Tuple of reflections for red, green, and blue light, each
                  ranging from 0.0 (no reflection) to 100.0 (high reflection).
        :rtype: (:ref:`percentage`, :ref:`percentage`, :ref:`percentage`)
        """
        pass


class InfraredSensor:
    """LEGO® MINDSTORMS® EV3 Infrared Sensor and Beacon."""

    def __init__(self, port):
        """InfraredSensor(port)

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def distance(self):
        """Measures the relative distance between the sensor and an object using
        infrared light.

        Returns:
            :ref:`relativedistance`: Relative distance ranging from 0 (closest)
            to 100 (farthest).

        """
        pass

    def beacon(self, channel):
        """Measures the relative distance and angle between the remote and the
        infrared sensor.

        Arguments:
            channel (int): Channel number of the remote.

        :returns: Tuple of relative distance (0 to 100) and approximate angle
                  (-75 to 75 degrees) between remote and infrared sensor.
        :rtype: (:ref:`relativedistance`, :ref:`angle`) or
                (``None``, ``None``) if no remote is detected.
        """
        pass

    def buttons(self, channel):
        """Checks which buttons on the infrared remote are pressed.

        This method can detect up to two buttons at once. If you press
        more buttons, you may not get useful data.

        Arguments:
            channel (int): Channel number of the remote.

        :returns: List of pressed buttons on the remote on selected channel.
        :rtype: List of :class:`Button <.parameters.Button>`

        """
        pass

    def keypad(self):
        """Checks which buttons on the infrared remote are pressed.

        This method can independently detect all 4 up/down buttons, but
        it cannot detect the beacon button.

        This method only works with the remote in channel 1.

        :returns: List of pressed buttons on the remote on selected channel.
        :rtype: List of :class:`Button <.parameters.Button>`

        """
        pass


class GyroSensor:
    """LEGO® MINDSTORMS® EV3 Gyro Sensor."""

    def __init__(self, port, positive_direction=_Direction.CLOCKWISE):
        """

        Arguments:
            port (Port): Port to which the sensor is connected.
            positive_direction (Direction):
                Positive rotation direction when looking at the red dot on top
                of the sensor.

        """
        pass

    def speed(self):
        """Gets the speed (angular velocity) of the sensor.

        Returns:
            :ref:`speed`: Sensor angular velocity.

        """
        pass

    def angle(self):
        """Gets the accumulated angle of the sensor.

        Returns:
            :ref:`angle`: Rotation angle.

        """
        pass

    def reset_angle(self, angle):
        """Sets the rotation angle of the sensor to a desired value.

        Arguments:
            angle (:ref:`angle`): Value to which the angle should be reset.
        """
        pass

    def _calibrate(self):
        """Calibrates the sensor.

        This process sets the speed and angle to zero and ensures that the
        angle value does not drift.

        Make sure that the sensor does not move while calibrating.

        This process can take up to 15 seconds.
        """
        pass


class UltrasonicSensor:
    """LEGO® MINDSTORMS® EV3 Ultrasonic Sensor."""

    def __init__(self, port):
        """UltrasonicSensor(port)

        Arguments:
            port (Port): Port to which the sensor is connected.

        """
        pass

    def distance(self, silent=False):
        """Measures the distance between the sensor and an object using
        ultrasonic sound waves.

        Arguments:
            silent (bool): Choose ``True`` to turn the sensor off after
                           measuring the distance. This reduces interference
                           with other ultrasonic sensors. If you do
                           this too frequently, the sensor can freeze.
                           If this happens, unplug it and plug it back in.

        Returns:
            :ref:`distance`: Distance.

        """
        pass

    def presence(self):
        """Checks for the presence of other ultrasonic sensors by detecting
        ultrasonic sounds.

        If the other ultrasonic sensor is operating in silent mode, you can
        only detect the presence of that sensor while it is taking a
        measurement.

        Returns:
            bool: ``True`` if ultrasonic sounds are detected,
            ``False`` if not.
        """
        pass



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from typing import Optional, Tuple

from .parameters import Port

class LUMPDevice:
    def __init__(self, port: Port): ...
    def read(self, mode: int) -> Tuple: ...
    def write(self, mode: int, values: Tuple): ...

class Ev3devSensor:
    sensor_index: int
    port_index: int
    def __init__(self, port: Port): ...
    def read(self, mode: int) -> Tuple: ...

class AnalogSensor:
    def __init__(self, port: Port): ...
    def voltage(self) -> int: ...
    def resistance(self) -> int: ...
    def active(self) -> None: ...
    def passive(self) -> None: ...

class I2CDevice:
    def __init__(self, port: Port, address: int): ...
    def read(self, reg: Optional[int], length: Optional[int] = 1) -> bytes: ...
    def write(self, reg: Optional[int], data: Optional[bytes] = None) -> None: ...

class UARTDevice:
    def __init__(self, port: Port, baudrate: int, timeout: Optional[int] = None): ...
    def read(self, length: int = 1) -> bytes: ...
    def read_all(self) -> bytes: ...
    def write(self, data: bytes) -> None: ...
    def waiting(self) -> int: ...
    def clear(self) -> None: ...



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pybricks', 'pybricks.media']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pybricks',
    'version': '2.0.0.post1',
    'description': 'Documentation and user-API stubs for Pybricks MicroPython',
    'long_description': "Pybricks end-user API & Documentation\n=====================================\n\nThis repository documents the Pybricks end-user MicroPython API.\n\nEach `Pybricks firmware`_ comes with the `Pybricks package`_. All modules,\nclasses, methods, and functions in that package have optimized implementations\nin C for high performance and reduced memory footprint.\n\nThis repository provides an empty, pure Python blueprint of that package. It is\nused to generate the `official documentation`_.\nSee the `contributor's guide <CONTRIBUTING.md>`_ for acceptable changes and\ninstructions to build the documentation locally.\n\nFor general discussion, please visit the `support`_ issue tracker.\n\n.. _Pybricks package: pybricks\n.. _Pybricks firmware: https://github.com/pybricks/pybricks-micropython\n.. _official documentation: https://docs.pybricks.com/\n.. _support: https://github.com/pybricks/support/issues\n",
    'author': 'The Pybricks Authors',
    'author_email': 'dev@pybricks.com',
    'maintainer': 'Laurens Valk',
    'maintainer_email': 'laurens@pybricks.com',
    'url': 'https://pybricks.com',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# SPDX-License-Identifier: MIT
# Copyright (c) 2020 The Pybricks Authors

from typing import Callable, Optional, Tuple

from ._common import ColorLight
from .iodevices import AnalogSensor
from .parameters import Color, Port

class TouchSensor:
    def __init__(self, port: Port): ...
    def pressed(self) -> bool: ...

class LightSensor:
    def __init__(self, port: Port): ...
    def ambient(self) -> int: ...
    def reflection(self) -> int: ...

class ColorSensor:
    light: ColorLight
    def __init__(self, port: Port): ...
    def color(self) -> Optional[Color]: ...
    def ambient(self) -> int: ...
    def reflection(self) -> int: ...
    def rgb(self) -> Tuple[int, int, int]: ...

class UltrasonicSensor:
    def __init__(self, port: Port): ...
    def distance(self) -> int: ...

class SoundSensor:
    def __init__(self, port: Port): ...
    def intensity(self, audible_only: bool = True) -> int: ...

class TemperatureSensor:
    def __init__(self, port: Port): ...
    def temperature(self) -> int: ...

class EnergyMeter:
    def __init__(self, port: Port): ...
    def storage(self) -> int: ...
    def input(self) -> Tuple[int, int, int]: ...
    def output(self) -> Tuple[int, int, int]: ...

class VernierAdapter(AnalogSensor):
    def __init__(
        self, port: Port, conversion: Optional[Callable[[int], float]] = None
    ): ...
    def voltage(self) -> int: ...
    def conversion(self, voltage: int) -> float: ...
    def value(self) -> float: ...



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""Robotics module for the Pybricks API."""

from ._common import Control as _Control


class DriveBase:
    """A robotic vehicle with two powered wheels and an optional support
    wheel or caster.

    By specifying the dimensions of your robot, this class
    makes it easy to drive a given distance in millimeters or turn by a given
    number of degrees.

    **Positive** distances and drive speeds mean
    driving **forward**. **Negative** means **backward**.

    **Positive** angles and turn rates mean turning **right**.
    **Negative** means **left**. So when viewed from the top,
    positive means clockwise and negative means counterclockwise.

    """

    distance_control = _Control()
    """The traveled distance and drive speed are controlled by a PID
    controller. You can use this attribute to change its settings.
    See :ref:`control` for an overview of available methods."""

    heading_control = _Control()
    """The robot turn angle and turn rate are controlled by a PID
    controller. You can use this attribute to change its settings.
    See :ref:`control` for an overview of available methods."""

    def __init__(self, left_motor, right_motor, wheel_diameter, axle_track):
        """DriveBase(left_motor, right_motor, wheel_diameter, axle_track)

        Arguments:
            left_motor (Motor):
                The motor that drives the left wheel.
            right_motor (Motor):
                The motor that drives the right wheel.
            wheel_diameter (:ref:`dimension`): Diameter of the wheels.
            axle_track (:ref:`dimension`): Distance between the points where
                both wheels touch the ground.
        """

    def drive(self, drive_speed, turn_rate):
        """Starts driving at the specified speed and turn rate. Both values are
        measured at the center point between the wheels of the robot.

        Arguments:
            drive_speed (:ref:`linspeed`): Speed of the robot.
            turn_rate (:ref:`speed`): Turn rate of the robot.
        """
        pass

    def stop(self):
        """Stops the robot by letting the motors spin freely."""
        pass

    def distance(self):
        """Gets the estimated driven distance.

        Returns:
            :ref:`distance`: Driven distance since last reset.
        """
        pass

    def angle(self):
        """Gets the estimated rotation angle of the drive base.

        Returns:
            :ref:`angle`: Accumulated angle since last reset.
        """
        pass

    def state(self):
        """Gets the state of the robot.

        This returns the current :meth:`.distance`, the drive speed, the
        :meth:`.angle`, and the turn rate.

        :returns: Distance, drive speed, angle, turn rate
        :rtype: (:ref:`distance`, :ref:`linspeed`, :ref:`angle`, :ref:`speed`)
        """
        pass

    def reset(self):
        """Resets the estimated driven distance and angle to 0."""
        pass

    def settings(self, straight_speed, straight_acceleration, turn_rate,
                 turn_acceleration):
        """Configures the speed and acceleration used
        by :meth:`.straight` and :meth:`.turn`.

        If you give no arguments, this returns the current values as a tuple.

        You can only change the settings while the robot is stopped. This is
        either before you begin driving or after you call :meth:`.stop`.

        Arguments:
            straight_speed (:ref:`linspeed`): Speed of the robot during
                :meth:`.straight`.
            straight_acceleration (:ref:`linacceleration`): Acceleration and
                deceleration of the robot at the start and end
                of :meth:`.straight`.
            turn_rate (:ref:`speed`): Turn rate of the robot
                during :meth:`.turn`.
            turn_acceleration (:ref:`acceleration`): Angular acceleration and
                deceleration of the robot at the start and end
                of :meth:`.turn`.
        """
        pass

    def straight(self, distance):
        """Drives straight for a given distance and then stops.

        Arguments:
            distance (:ref:`distance`): Distance to travel.
        """
        pass

    def turn(self, angle):
        """Turns in place by a given angle and then stops.

        Arguments:
            angle (:ref:`angle`): Angle of the turn.
        """
        pass



# SPDX-License-Identifier: MIT
# Copyright (c) 2018-2020 The Pybricks Authors

"""
Classes to exchange messages between EV3 bricks.
"""


class Mailbox:
    def __init__(self, name, connection, encode=None, decode=None):
        """Object that represents a mailbox containing data.

        You can read data that is delivered by other EV3 bricks, or send data
        to other bricks that have the same mailbox.

        By default, the mailbox reads and send only bytes. To send other
        data, you can provide an ``encode`` function that encodes your Python
        object into bytes, and a ``decode`` function to convert bytes back to
        a Python object.

        Arguments:
            name (str):
                The name of this mailbox.
            connection:
                A connection object such as :class:`BluetoothMailboxClient`.
            encode (callable):
                Function that encodes a Python object to bytes.
            decode (callable):
                Function that creates a new Python object from bytes.
        """

    def read(self):
        """Gets the current value of the mailbox.

        Returns:
            The current value or ``None`` if the mailbox is empty.
        """
        return ''

    def send(self, value, brick=None):
        """Sends a value to this mailbox on connected devices.

        Arguments:
            value:
                The value that will be delivered to the mailbox.
            brick (str):
                The name or Bluetooth address of the brick or ``None`` to
                to broadcast to all connected devices.

        Raises:
            OSError:
                There is a problem with the connection.
        """

    def wait(self):
        """Waits for the mailbox to be updated by remote device."""

    def wait_new(self):
        """Waits for a new value to be delivered to the mailbox that is not
        equal to the current value in the mailbox.


        Returns:
            The new value.
        """
        return object()


class LogicMailbox(Mailbox):
    def __init__(self, name, connection):
        """Object that represents a mailbox containing boolean data.

        This works just like a regular :class:`Mailbox`, but values
        must be ``True`` or ``False``.

        This is compatible with the "logic" mailbox type in EV3-G.

        Arguments:
            name (str):
                The name of this mailbox.
            connection:
                A connection object such as :class:`BluetoothMailboxClient`.
        """


class NumericMailbox(Mailbox):
    def __init__(self, name, connection):
        """Object that represents a mailbox containing numeric data.

        This works just like a regular :class:`Mailbox`, but values must be a
        number, such as ``15`` or ``12.345``

        This is compatible with the "numeric" mailbox type in EV3-G.

        Arguments:
            name (str):
                The name of this mailbox.
            connection:
                A connection object such as :class:`BluetoothMailboxClient`.
        """


class TextMailbox(Mailbox):
    def __init__(self, name, connection):
        """Object that represents a mailbox containing text data.

        This works just like a regular :class:`Mailbox`, but data must be a
        string, such as ``'hello!'`` or ``'My name is EV3'``.

        This is compatible with the "text" mailbox type in EV3-G.

        Arguments:
            name (str):
                The name of this mailbox.
            connection:
                A connection object such as :class:`BluetoothMailboxClient`.
        """


class BluetoothMailboxServer:
    """Object that represents a Bluetooth connection from one or more remote
    EV3s.

    The remote EV3s can either be running MicroPython or the standard EV3
    firmware.

    A "server" waits for a "client" to connect to it.
    """

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def wait_for_connection(self, count=1):
        """Waits for a :class:`BluetoothMailboxClient` on a remote device to
        connect.

        Arguments:
            count (int):
                The number of remote connections to wait for.

        Raises:
            OSError:
                There was a problem establishing the connection.
        """

    def close(self):
        """Closes all connections."""


class BluetoothMailboxClient:
    """Object that represents a Bluetooth connection to one or more remote EV3s.

    The remote EV3s can either be running MicroPython or the standard EV3
    firmware.

    A "client" initiates a connection to a waiting "server".
    """

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def connect(self, brick):
        """Connects to an :class:`BluetoothMailboxServer` on another device.

        The remote device must be paired and waiting for a connection. See
        :meth:`BluetoothMailboxServer.wait_for_connection`.

        Arguments:
            brick (str):
                The name or Bluetooth address of the remote EV3 to connect to.

        Raises:
            OSError:
                There was a problem establishing the connection.
        """

    def close(self):
        """Closes all connections."""
